﻿using System;
using ZWave.ZnifferApplication.Enums;
using Utils;
using Utils.Threading;

namespace ZWave.ZnifferApplication.Operations
{
    public class CollectDataOperation : ActionBase
    {
        public delegate bool DataItemPredicateDelegate(ActionToken actionToken, DataItem dataIem);
        private byte[] HomeId { get; set; }
        private byte? SrcNodeId { get; set; }
        protected DataItemPredicateDelegate DataItemPredicate { get; set; }
        private int TimeoutMs { get; set; }
        public CollectDataOperation(byte[] homeId, byte? srcNodeId, DataItemPredicateDelegate dataItemPredicate, int timeoutMs) :
            base(false)
        {
            HomeId = homeId;
            SrcNodeId = srcNodeId;
            DataItemPredicate = dataItemPredicate;
            TimeoutMs = timeoutMs;
        }

        private ZnifferApiHandler Handler { get; set; }

        protected override void CreateWorkflow()
        {
            ActionUnits.Add(new StartActionUnit(null, TimeoutMs));
            ActionUnits.Add(new DataReceivedUnit(Handler, OnReceived));
        }

        protected override void CreateInstance()
        {
            Handler = new ZnifferApiHandler(CommandTypes.DataHandler);
            if (HomeId != null && HomeId.Length >= 4)
            {
                if (SrcNodeId != null)
                {
                    Handler.AddConditions(
                        new ByteIndex(HomeId[0]),
                        new ByteIndex(HomeId[1]),
                        new ByteIndex(HomeId[2]),
                        new ByteIndex(HomeId[3]),
                        new ByteIndex((byte)SrcNodeId));
                }
                else
                {
                    Handler.AddConditions(
                        new ByteIndex(HomeId[0]),
                        new ByteIndex(HomeId[1]),
                        new ByteIndex(HomeId[2]),
                        new ByteIndex(HomeId[3]));
                }
            }
            else
            {
                if (SrcNodeId != null)
                {
                    Handler.AddConditions(
                        ByteIndex.AnyValue,
                        ByteIndex.AnyValue,
                        ByteIndex.AnyValue,
                        ByteIndex.AnyValue,
                        new ByteIndex((byte)SrcNodeId));
                }
            }
        }

        protected virtual void OnReceived(DataReceivedUnit ou)
        {
            if (Token.IsStateActive && SpecificResult.DataItems.Count < SpecificResult.MaxCapacity)
            {
                DataItem dataItem = ((DataFrame)ou.DataFrame).DataItem;
                if (DataItemPredicate != null)
                {
                    if (DataItemPredicate(Token, dataItem))
                    {
                        SpecificResult.DataItems.Add(dataItem);
                    }
                }
            }
        }

        public CollectDataResult SpecificResult
        {
            get { return (CollectDataResult)Result; }
        }

        protected override ActionResult CreateOperationResult()
        {
            return new CollectDataResult(10000);
        }
    }

    public class CollectDataResult : ActionResult
    {
        public int MaxCapacity { get; set; }
        public ConcurrentList<DataItem> DataItems { get; set; }
        public CollectDataResult(int maxCapacity)
        {
            MaxCapacity = maxCapacity;
            DataItems = new ConcurrentList<DataItem>(MaxCapacity);
        }

        public ConcurrentList<DataItem> ReturnAndReset()
        {
            return DataItems.ReturnAndReset();
        }
    }
}
